@@ -3,7 +3,7 @@ from ctypes import Structure, LittleEndianStructure, \
                    CFUNCTYPE, POINTER, addressof, sizeof, cast, \
                    c_short, c_int, c_uint, c_size_t, c_long, \
                    c_uint8, c_uint16, c_uint32, \
-                   c_void_p, c_char_p, py_object, string_at
+                   c_void_p, c_char_p, py_object, string_at, pointer
 try:
     from ctypes import c_ssize_t
 except ImportError:
@@ -22,14 +22,15 @@ import ctypes.util
 import platform
 import os.path
 import sys
+import inspect
 
 class Enum(object):
     def __init__(self, member_dict):
         forward_dict = {}
         reverse_dict = {}
-        module_globals = globals()
+        global_dict = inspect.stack()[1][0].f_globals
         next_value = 0
-        for name, value in member_dict.iteritems():
+        for name, value in member_dict.items():
             if value is None:
                 value = next_value
                 next_value += 1
@@ -38,7 +39,7 @@ class Enum(object):
                 raise ValueError('Multiple names for value %r: %r, %r' %
                     (value, reverse_dict[value], name))
             reverse_dict[value] = name
-            module_globals[name] = value
+            global_dict[name] = value
         self.forward_dict = forward_dict
         self.reverse_dict = reverse_dict
 
@@ -99,6 +100,13 @@ class USBError(Exception):
         return '%s [%s]' % (libusb_error.get(self.value, 'Unknown error'),
                             self.value)
 
+if sys.version_info[0] == 3:
+    _string_item_to_int = lambda x: x
+    _empty_char_p = bytes()
+else:
+    _string_item_to_int = ord
+    _empty_char_p = ''
+
 c_uchar = c_uint8
 c_int_p = POINTER(c_int)
 
@@ -491,6 +499,16 @@ class libusb_device_handle(Structure):
 libusb_device_handle_p = POINTER(libusb_device_handle)
 libusb_device_handle_p_p = POINTER(libusb_device_handle_p)
 
+class libusb_version(Structure):
+    _fields_ = [
+        ('major', c_uint16),
+        ('minor', c_uint16),
+        ('micro', c_uint16),
+        ('nano', c_uint16),
+        ('rc', c_char_p),
+        ('describe', c_char_p),
+    ]
+
 libusb_speed = Enum({
 # The OS doesn't report or know the device speed.
 'LIBUSB_SPEED_UNKNOWN': 0,
@@ -601,7 +619,7 @@ _libusb_transfer_fields = [
     ('num_iso_packets', c_int),
     ('iso_packet_desc', libusb_iso_packet_descriptor)
 ]
-if platform.system() == 'FreeBSD' and getattr(libusb,
+if 'FreeBSD' in platform.system() and getattr(libusb,
         'libusb_get_string_descriptor', None) is None:
     # Old FreeBSD version has a slight ABI incompatibility.
     # Work around it unless libusb_get_string_descriptor is available, as it
@@ -629,6 +647,17 @@ libusb_exit.restype = None
 libusb_set_debug = libusb.libusb_set_debug
 libusb_set_debug.argtypes = [libusb_context_p, c_int]
 libusb_set_debug.restype = None
+#const struct libusb_version * libusb_get_version(void);
+try:
+    libusb_get_version = libusb.libusb_get_version
+except AttributeError:
+    _dummy_version = libusb_version(0, 0, 0, 0, _empty_char_p, _empty_char_p)
+    _dummy_version_p = pointer(_dummy_version)
+    def libusb_get_version():
+        return _dummy_version_p
+else:
+    libusb_get_version.argtypes = []
+    libusb_get_version.restype = POINTER(libusb_version)
 #int libusb_has_capability(uint32_t capability);
 try:
     libusb_has_capability = libusb.libusb_has_capability
@@ -637,6 +666,7 @@ except AttributeError:
         return 0
 else:
     libusb_has_capability.argtypes = [c_uint32]
+    libusb_has_capability.restype = c_int
 try:
     # Note: Should be equivalent to libusb_error.get (except libusb_error.get
     # one raises on unknown values).
@@ -923,7 +953,7 @@ def get_extra(descriptor):
         extra = string_at(descriptor.extra, extra_length)
         append = result.append
         while extra:
-            length = ord(extra[0])
+            length = _string_item_to_int(extra[0])
             if not (0 < length <= len(extra)):
                 raise ValueError('Extra descriptor %i is incomplete/invalid' % (
                     len(result), ))
