@@ -1,12 +1,12 @@
 """
 Pythonic wrapper for libusb-1.0.
 
-The first thing you must do is to get an "USB context". To do so, create a
-LibUSBContext instance.
+The first thing you must do is to get an "USB context". To do so, create an
+USBContext instance.
 Then, you can use it to browse available USB devices and open the one you want
 to talk to.
 At this point, you should have a USBDeviceHandle instance (as returned by
-LibUSBContext or USBDevice instances), and you can start exchanging with the
+USBContext or USBDevice instances), and you can start exchanging with the
 device.
 
 Features:
@@ -21,16 +21,35 @@ Features:
 
 import libusb1
 from ctypes import byref, create_string_buffer, c_int, sizeof, POINTER, \
-    create_unicode_buffer, c_wchar, cast, c_uint16, c_ubyte, string_at, \
-    c_void_p, cdll
+    cast, c_uint16, c_ubyte, string_at, c_void_p, cdll
 import sys
 import threading
 from ctypes.util import find_library
 import warnings
 import weakref
+import collections
+#import select
+try:
+    namedtuple = collections.namedtuple
+except AttributeError:
+    Version = lambda *x: x
+else:
+    Version = namedtuple('Version', ['major', 'minor', 'micro', 'nano', 'rc',
+        'describe'])
+
+if sys.version_info[0] == 3:
+    BYTE = bytes([0])
+    xrange = range
+    long = int
+else:
+    BYTE = '\x00'
+CONTROL_SETUP = BYTE * libusb1.LIBUSB_CONTROL_SETUP_SIZE
 
 __all__ = ['USBContext', 'USBDeviceHandle', 'USBDevice',
-    'USBPoller', 'USBTransfer', 'USBTransferHelper', 'EVENT_CALLBACK_SET']
+    'USBPoller', 'USBTransfer', 'USBTransferHelper', 'EVENT_CALLBACK_SET',
+    'USBPollerThread', 'USBEndpoint', 'USBInterfaceSetting', 'USBInterface',
+    'USBConfiguration', 'DoomedTransferError', 'getVersion',
+]
 
 if sys.version_info[:2] >= (2, 6):
     if sys.platform == 'win32':
@@ -90,6 +109,7 @@ def create_binary_buffer(string_or_len):
     return result
 
 class DoomedTransferError(Exception):
+    """Exception raised when altering/submitting a doomed transfer."""
     pass
 
 class USBTransfer(object):
@@ -184,8 +204,8 @@ class USBTransfer(object):
                 # reference to transfer, so a segfault might happen anyway.
                 # Should we warn user ? How ?
                 self.cancel()
-            except self.__USBError, exception:
-                if exception.value == self.__LIBUSB_ERROR_NOT_FOUND:
+            except self.__USBError:
+                if sys.exc_info()[1].value == self.__LIBUSB_ERROR_NOT_FOUND:
                     # Transfer was not submitted, we can free it.
                     self.__libusb_free_transfer(self.__transfer)
                 else:
@@ -208,8 +228,6 @@ class USBTransfer(object):
         """
         Change transfer's callback.
         """
-        if self.__submitted:
-            raise ValueError('Cannot alter a submitted transfer')
         self.__callback = callback
 
     def getCallback(self):
@@ -223,27 +241,32 @@ class USBTransfer(object):
         """
         Setup transfer for control use.
 
-        request_type, request, value, index: See USBDeviceHandle.controlWrite.
-        buffer_or_len: either a string (when sending data), or expected data
-          length (when receiving data)
-        callback: function to call upon event. Called with transfer as
-          parameter, return value ignored.
-        user_data: to pass some data to/from callback
-        timeout: in milliseconds, how long to wait for devices acknowledgement
-          or data. Set to 0 to disable.
+        request_type, request, value, index
+            See USBDeviceHandle.controlWrite.
+            request_type defines transfer direction (see
+            libusb1.LIBUSB_ENDPOINT_OUT and libusb1.LIBUSB_ENDPOINT_IN)).
+        buffer_or_len
+            Either a string (when sending data), or expected data length (when
+            receiving data).
+        callback
+            Callback function to be invoked on transfer completion.
+            Called with transfer as parameter, return value ignored.
+        user_data
+            User data to pass to callback function.
+        timeout
+            Transfer timeout in milliseconds. 0 to disable.
         """
         if self.__submitted:
             raise ValueError('Cannot alter a submitted transfer')
         if self.__doomed:
             raise DoomedTransferError('Cannot reuse a doomed transfer')
-        if isinstance(buffer_or_len, basestring):
-            length = len(buffer_or_len)
-            string_buffer = create_binary_buffer(
-                ' ' * libusb1.LIBUSB_CONTROL_SETUP_SIZE + buffer_or_len)
-        else:
+        if isinstance(buffer_or_len, (int, long)):
             length = buffer_or_len
             string_buffer = create_binary_buffer(length + \
                 libusb1.LIBUSB_CONTROL_SETUP_SIZE)
+        else:
+            length = len(buffer_or_len)
+            string_buffer = create_binary_buffer(CONTROL_SETUP + buffer_or_len)
         self.__initialized = False
         self.__transfer_buffer = string_buffer
         self.__user_data = user_data
@@ -259,15 +282,19 @@ class USBTransfer(object):
         """
         Setup transfer for bulk use.
 
-        endpoint: endpoint to submit transfer to (implicitly sets transfer
-          direction).
-        buffer_or_len: either a string (when sending data), or expected data
-          length (when receiving data)
-        callback: function to call upon event. Called with transfer as
-          parameter, return value ignored.
-        user_data: to pass some data to/from callback
-        timeout: in milliseconds, how long to wait for devices acknowledgement
-          or data. Set to 0 to disable.
+        endpoint
+            Endpoint to submit transfer to. Defines transfer direction (see
+            libusb1.LIBUSB_ENDPOINT_OUT and libusb1.LIBUSB_ENDPOINT_IN)).
+        buffer_or_len
+            Either a string (when sending data), or expected data length (when
+            receiving data)
+        callback
+            Callback function to be invoked on transfer completion.
+            Called with transfer as parameter, return value ignored.
+        user_data
+            User data to pass to callback function.
+        timeout
+            Transfer timeout in milliseconds. 0 to disable.
         """
         if self.__submitted:
             raise ValueError('Cannot alter a submitted transfer')
@@ -288,15 +315,19 @@ class USBTransfer(object):
         """
         Setup transfer for interrupt use.
 
-        endpoint: endpoint to submit transfer to (implicitly sets transfer
-          direction).
-        buffer_or_len: either a string (when sending data), or expected data
-          length (when receiving data)
-        callback: function to call upon event. Called with transfer as
-          parameter, return value ignored.
-        user_data: to pass some data to/from callback
-        timeout: in milliseconds, how long to wait for devices acknowledgement
-          or data. Set to 0 to disable.
+        endpoint
+            Endpoint to submit transfer to. Defines transfer direction (see
+            libusb1.LIBUSB_ENDPOINT_OUT and libusb1.LIBUSB_ENDPOINT_IN)).
+        buffer_or_len
+            Either a string (when sending data), or expected data length (when
+            receiving data)
+        callback
+            Callback function to be invoked on transfer completion.
+            Called with transfer as parameter, return value ignored.
+        user_data
+            User data to pass to callback function.
+        timeout
+            Transfer timeout in milliseconds. 0 to disable.
         """
         if self.__submitted:
             raise ValueError('Cannot alter a submitted transfer')
@@ -317,20 +348,23 @@ class USBTransfer(object):
         """
         Setup transfer for isochronous use.
 
-        endpoint: endpoint to submit transfer to (implicitly sets transfer
-          direction).
-        buffer_or_len: either a string (when sending data), or expected data
-          length (when receiving data)
-        callback: function to call upon event. Called with transfer as
-          parameter, return value ignored.
-        user_data: to pass some data to/from callback
-        timeout: in milliseconds, how long to wait for devices acknowledgement
-          or data. Set to 0 to disable.
-        iso_transfer_length_list: list of individual transfer sizes. If not
-          provided, buffer_or_len's size will be divided evenly among the
-          number of ISO transfers given to receive current instance, rounded
-          down. Providing a list allows overriding this (both the number of
-          ISO transfers and their individual lengths).
+        endpoint
+            Endpoint to submit transfer to. Defines transfer direction (see
+            libusb1.LIBUSB_ENDPOINT_OUT and libusb1.LIBUSB_ENDPOINT_IN)).
+        buffer_or_len
+            Either a string (when sending data), or expected data length (when
+            receiving data)
+        callback
+            Callback function to be invoked on transfer completion.
+            Called with transfer as parameter, return value ignored.
+        user_data
+            User data to pass to callback function.
+        timeout
+            Transfer timeout in milliseconds. 0 to disable.
+        iso_transfer_length_list
+            List of individual transfer sizes. If not provided, buffer_or_len
+            will be divided evenly among available transfers if possible, and
+            raise ValueError otherwise.
         """
         if self.__submitted:
             raise ValueError('Cannot alter a submitted transfer')
@@ -344,7 +378,11 @@ class USBTransfer(object):
         string_buffer = create_binary_buffer(buffer_or_len)
         buffer_length = sizeof(string_buffer)
         if iso_transfer_length_list is None:
-            iso_length = buffer_length // num_iso_packets
+            iso_length, remainder = divmod(buffer_length, num_iso_packets)
+            if remainder:
+                raise ValueError('Buffer size %i cannot be evenly '
+                    'distributed among %i transfers' % (buffer_length,
+                    num_iso_packets))
             iso_transfer_length_list = [iso_length] * num_iso_packets
         configured_iso_packets = len(iso_transfer_length_list)
         if configured_iso_packets > num_iso_packets:
@@ -421,8 +459,6 @@ class USBTransfer(object):
         """
         Change user data.
         """
-        if self.__submitted:
-            raise ValueError('Cannot alter a submitted transfer')
         self.__user_data = user_data
 
     def getISOBufferList(self):
@@ -695,8 +731,8 @@ class USBPollerThread(threading.Thread):
                         if poll(getNextTimeout()):
                             try:
                                 handle_events_locked()
-                            except libusb1.USBError, exc:
-                                exceptionHandler(exc)
+                            except libusb1.USBError:
+                                exceptionHandler(sys.exc_info()[1])
                 finally:
                     unlock_events()
 
@@ -708,6 +744,37 @@ class USBPollerThread(threading.Thread):
         self.__fd_set.discard(fd)
         self.__poller.unregister(fd)
 
+#class KQueuePoller(object):
+#    """
+#    Minmal kqueue wrapper to expose poll-ish API.
+#    """
+#    def __init__(self, kqueue):
+#        self._kqueue = kqueue
+#        self._fd_set = set()
+#
+#    def register(self, fd, events):
+#        kevent = select.kevent(fd, flags=select.KQ_EV_ADD)
+#        # It's needed to duplicate poll event values, so this works when they
+#        # are not available from select module.
+#        if events & :
+#            kevent.filter = select.KQ_FILTER_READ
+#            self._kqueue.control()
+#        if events & :
+#            kevent.filter = select.KQ_FILTER_WRITE
+#        self._kqueue.control([select.kevent(fd, kq_events, select.KQ_EV_ADD)], 0)
+#        # fd come from libusb, they are file descriptors, not python objects
+#        assert isinstance(fd, int), fd
+#        self._fd_set.add(fd)
+#
+#    def unregister(self, fd):
+#        self._kqueue.control([select.kevent(fd, 0, select.KQ_EV_DELETE)], 0)
+#        # fd come from libusb, they are file descriptors, not python objects
+#        assert isinstance(fd, int), fd
+#        self._fd_set.remove(fd)
+#
+#    def poll(self, timeout=None):
+#        self._kqueue.control(None, len(self._fd_set), timeout * 1000.)
+
 class USBPoller(object):
     """
     Class allowing integration of USB event polling in a file-descriptor
@@ -868,8 +935,9 @@ class USBDeviceHandle(object):
             for transfer in inflight:
                 try:
                     transfer.cancel()
-                except self.__USBError, exception:
-                    if exception.value != self.__LIBUSB_ERROR_NOT_FOUND:
+                except self.__USBError:
+                    if sys.exc_info()[1].value != \
+                            self.__LIBUSB_ERROR_NOT_FOUND:
                         raise
             while inflight:
                 self.__context.handleEvents()
@@ -894,7 +962,7 @@ class USBDeviceHandle(object):
                                                   byref(configuration))
         if result:
             raise libusb1.USBError(result)
-        return configuration
+        return configuration.value
 
     def setConfiguration(self, configuration):
         """
@@ -984,6 +1052,11 @@ class USBDeviceHandle(object):
         """
         Return a list of USB language identifiers (as integers) supported by
         current device for its string descriptors.
+
+        Note: language identifiers seem (I didn't check them all...) very
+        similar to windows language identifiers, so you may want to use
+        locales.windows_locale to get an rfc3066 representation. The 5 standard
+        HID language codes are missing though.
         """
         descriptor_string = create_binary_buffer(STRING_LENGTH)
         result = libusb1.libusb_get_string_descriptor(self.__handle,
@@ -1009,15 +1082,14 @@ class USBDeviceHandle(object):
         Return value is an unicode string.
         Return None if there is no such descriptor on device.
         """
-        descriptor_string = create_unicode_buffer(
-            STRING_LENGTH / sizeof(c_wchar))
+        descriptor_string = create_binary_buffer(STRING_LENGTH)
         result = libusb1.libusb_get_string_descriptor(self.__handle,
             descriptor, lang_id, descriptor_string, sizeof(descriptor_string))
         if result == libusb1.LIBUSB_ERROR_NOT_FOUND:
             return None
         if result < 0:
             raise libusb1.USBError(result)
-        return descriptor_string.value
+        return descriptor_string.value.decode('UTF-16-LE')
 
     def getASCIIStringDescriptor(self, descriptor):
         """
@@ -1033,7 +1105,7 @@ class USBDeviceHandle(object):
             return None
         if result < 0:
             raise libusb1.USBError(result)
-        return descriptor_string.value
+        return descriptor_string.value.decode('ASCII')
 
     # Sync I/O
 
@@ -1171,6 +1243,10 @@ class USBDeviceHandle(object):
 
 class USBConfiguration(object):
     def __init__(self, config):
+        """
+        You should not instanciate this class directly.
+        Call USBDevice methods to get instances of this class.
+        """
         if not isinstance(config, libusb1.libusb_config_descriptor):
             raise TypeError('Unexpected descriptor type.')
         self.__config = config
@@ -1188,17 +1264,35 @@ class USBConfiguration(object):
         return self.__config.bmAttributes
 
     def getMaxPower(self):
+        """
+        Returns device's power consumption in mW.
+        Beware of unit: USB descriptor uses 2mW increments, this method
+        converts it to mW units.
+        """
         return self.__config.MaxPower * 2
 
     def getExtra(self):
+        """
+        Returns a list of extra (non-basic) descriptors (DFU, HID, ...).
+        """
         return libusb1.get_extra(self.__config)
 
-    def iterInterfaces(self):
+    def __iter__(self):
+        """
+        Iterates over interfaces available in this configuration, yielding
+        USBInterface instances.
+        """
         interface_list = self.__config.interface
         for interface_num in xrange(self.getNumInterfaces()):
             yield USBInterface(interface_list[interface_num])
 
+    # BBB
+    iterInterfaces = __iter__
+
     def __getitem__(self, interface):
+        """
+        Returns an USBInterface instance.
+        """
         if not isinstance(interface, int):
             raise TypeError('interface parameter must be an integer')
         if not (0 <= interface < self.getNumInterfaces()):
@@ -1207,6 +1301,10 @@ class USBConfiguration(object):
 
 class USBInterface(object):
     def __init__(self, interface):
+        """
+        You should not instanciate this class directly.
+        Call USBConfiguration methods to get instances of this class.
+        """
         if not isinstance(interface, libusb1.libusb_interface):
             raise TypeError('Unexpected descriptor type.')
         self.__interface = interface
@@ -1214,12 +1312,22 @@ class USBInterface(object):
     def getNumSettings(self):
         return self.__interface.num_altsetting
 
-    def iterSettings(self):
+    def __iter__(self):
+        """
+        Iterates over settings in this insterface, yielding
+        USBInterfaceSetting instances.
+        """
         alt_setting_list = self.__interface.altsetting
         for alt_setting_num in xrange(self.getNumSettings()):
             yield USBInterfaceSetting(alt_setting_list[alt_setting_num])
 
+    # BBB
+    iterSettings = __iter__
+
     def __getitem__(self, alt_setting):
+        """
+        Returns an USBInterfaceSetting instance.
+        """
         if not isinstance(alt_setting, int):
             raise TypeError('alt_setting parameter must be an integer')
         if not (0 <= alt_setting < self.getNumSettings()):
@@ -1228,6 +1336,10 @@ class USBInterface(object):
 
 class USBInterfaceSetting(object):
     def __init__(self, alt_setting):
+        """
+        You should not instanciate this class directly.
+        Call USBDevice or USBInterface methods to get instances of this class.
+        """
         if not isinstance(alt_setting, libusb1.libusb_interface_descriptor):
             raise TypeError('Unexpected descriptor type.')
         self.__alt_setting = alt_setting
@@ -1264,12 +1376,22 @@ class USBInterfaceSetting(object):
     def getExtra(self):
         return libusb1.get_extra(self.__alt_setting)
 
-    def iterEndpoints(self):
+    def __iter__(self):
+        """
+        Iterates over endpoints in this interface setting , yielding
+        USBEndpoint instances.
+        """
         endpoint_list = self.__alt_setting.endpoint
         for endpoint_num in xrange(self.getNumEndpoints()):
             yield USBEndpoint(endpoint_list[endpoint_num])
 
+    # BBB
+    iterEndpoints = __iter__
+
     def __getitem__(self, endpoint):
+        """
+        Returns an USBEndpoint instance.
+        """
         if not isinstance(endpoint, int):
             raise TypeError('endpoint parameter must be an integer')
         if not (0 <= endpoint < self.getNumEndpoints()):
@@ -1308,7 +1430,7 @@ class USBDevice(object):
     Represents a USB device.
     """
 
-    __configuration_descriptor_list = None
+    __configuration_descriptor_list = ()
     __libusb_unref_device = libusb1.libusb_unref_device
     __libusb_free_config_descriptor = libusb1.libusb_free_config_descriptor
     __byref = byref
@@ -1347,10 +1469,9 @@ class USBDevice(object):
 
     def __del__(self):
         self.__libusb_unref_device(self.device_p)
-        if self.__configuration_descriptor_list is not None:
-            byref = self.__byref
-            for config in self.__configuration_descriptor_list:
-                self.__libusb_free_config_descriptor(byref(config))
+        byref = self.__byref
+        for config in self.__configuration_descriptor_list:
+            self.__libusb_free_config_descriptor(byref(config))
 
     def __str__(self):
         return 'Bus %03i Device %03i: ID %04x:%04x' % (
@@ -1363,14 +1484,17 @@ class USBDevice(object):
     def __getitem__(self, index):
         return USBConfiguration(self.__configuration_descriptor_list[index])
 
-    def iterConfiguations(self):
+    def iterConfigurations(self):
         for config in self.__configuration_descriptor_list:
             yield USBConfiguration(config)
 
+    # BBB
+    iterConfiguations = iterConfigurations
+
     def iterSettings(self):
         for config in self.__configuration_descriptor_list:
-            for interface in USBConfiguration(config).iterInterfaces():
-                for setting in interface.iterSettings():
+            for interface in USBConfiguration(config):
+                for setting in interface:
                     yield setting
 
     def getBusNumber(self):
@@ -1418,6 +1542,10 @@ class USBDevice(object):
     def getMaxPacketSize(self, endpoint):
         """
         Get device's max packet size for given endpoint.
+
+        Warning: this function will not always give you the expected result.
+        See https://libusb.org/ticket/77 . You should instead consult the
+        endpoint descriptor of current configuration and alternate setting.
         """
         result = libusb1.libusb_get_max_packet_size(self.device_p, endpoint)
         if result < 0:
@@ -1428,6 +1556,10 @@ class USBDevice(object):
         """
         Get the maximum size for a single isochronous packet for given
         endpoint.
+
+        Warning: this function will not always give you the expected result.
+        See https://libusb.org/ticket/77 . You should instead consult the
+        endpoint descriptor of current configuration and alternate setting.
         """
         result = libusb1.libusb_get_max_iso_packet_size(self.device_p, endpoint)
         if result < 0:
@@ -1580,7 +1712,7 @@ class USBContext(object):
         """
         Close (destroy) this USB context.
 
-        When this function has been called, methods on its instance will
+        When this method has been called, methods on its instance will
         become mosty no-ops, returning None.
         """
         self.__context_cond.acquire()
@@ -1601,50 +1733,57 @@ class USBContext(object):
             self.__removed_cb = None
 
     @_validContext
-    def getDeviceList(self, skip_on_access_error=False):
+    def getDeviceList(self, skip_on_access_error=False, skip_on_error=False):
         """
         Return a list of all USB devices currently plugged in, as USBDevice
         instances.
 
+        skip_on_error (bool)
+            If True, ignore devices which raise USBError.
+
         skip_on_access_error (bool)
-            If True, catch LIBUSB_ERROR_ACCESS errors and just skip to next
-            device.
+            DEPRECATED. Alias for skip_on_error.
         """
+        skip_on_error = skip_on_error or skip_on_access_error
         device_p_p = libusb1.libusb_device_p_p()
         libusb_device_p = libusb1.libusb_device_p
         device_list_len = libusb1.libusb_get_device_list(self.__context_p,
                                                          byref(device_p_p))
         if device_list_len < 0:
             raise libusb1.USBError(device_list_len)
-        result = []
-        append = result.append
-        for device_p in device_p_p[:device_list_len]:
-            try:
-                # Instanciate our own libusb_device_p object so we can free
-                # libusb-provided device list. Is this a bug in ctypes that it
-                # doesn't copy pointer value (=pointed memory address) ? At
-                # least, it's not so convenient and forces using such weird
-                # code.
-                device = USBDevice(self, libusb_device_p(device_p.contents))
-            except libusb1.USBError, exc:
-                if exc.value != libusb1.LIBUSB_ERROR_ACCESS or \
-                        not skip_on_access_error:
-                    raise
-            else:
-                append(device)
-        libusb1.libusb_free_device_list(device_p_p, 0)
+        try:
+            result = []
+            append = result.append
+            for device_p in device_p_p[:device_list_len]:
+                try:
+                    # Instanciate our own libusb_device_p object so we can free
+                    # libusb-provided device list. Is this a bug in ctypes that it
+                    # doesn't copy pointer value (=pointed memory address) ? At
+                    # least, it's not so convenient and forces using such weird
+                    # code.
+                    device = USBDevice(self, libusb_device_p(device_p.contents))
+                except libusb1.USBError:
+                    if not skip_on_error:
+                        raise
+                else:
+                    append(device)
+        finally:
+            libusb1.libusb_free_device_list(device_p_p, 0)
         return result
 
     def getByVendorIDAndProductID(self, vendor_id, product_id,
-            skip_on_access_error=False):
+            skip_on_access_error=False, skip_on_error=False):
         """
         Get the first USB device matching given vendor and product ids.
         Returns an USBDevice instance, or None if no present device match.
+        skip_on_error (bool)
+            (see getDeviceList)
         skip_on_access_error (bool)
             (see getDeviceList)
         """
         for device in self.getDeviceList(
-                skip_on_access_error=skip_on_access_error):
+                skip_on_access_error=skip_on_access_error,
+                skip_on_error=skip_on_error):
             if device.getVendorID() == vendor_id and \
                     device.getProductID() == product_id:
                 result = device
@@ -1654,16 +1793,19 @@ class USBContext(object):
         return result
 
     def openByVendorIDAndProductID(self, vendor_id, product_id,
-            skip_on_access_error=False):
+            skip_on_access_error=False, skip_on_error=False):
         """
         Get the first USB device matching given vendor and product ids.
         Returns an USBDeviceHandle instance, or None if no present device
         match.
+        skip_on_error (bool)
+            (see getDeviceList)
         skip_on_access_error (bool)
             (see getDeviceList)
         """
         result = self.getByVendorIDAndProductID(vendor_id, product_id,
-            skip_on_access_error=skip_on_access_error)
+            skip_on_access_error=skip_on_access_error,
+            skip_on_error=skip_on_error)
         if result is not None:
             result = result.open()
         return result
@@ -1846,8 +1988,31 @@ class USBContext(object):
         """
         return libusb1.libusb_event_handler_active(self.__context_p)
 
+    def hasCapability(self, capability):
+        """
+        Tests feature presence.
+        See libusb1.libusb_capability .
+        """
+        return libusb1.libusb_has_capability(capability)
+
 del USBContext._validContext
 
+def getVersion():
+    """
+    Returns underlying libusb's version information as a 6-namedtuple (or
+    6-tuple if namedtuples are not avaiable):
+    - major
+    - minor
+    - micro
+    - nano
+    - rc
+    - describe
+    Returns (0, 0, 0, 0, '', '') if libusb doesn't have required entry point.
+    """
+    version = libusb1.libusb_get_version().contents
+    return Version(version.major, version.minor, version.micro, version.nano,
+        version.rc, version.describe)
+
 class LibUSBContext(USBContext):
     """
     Backward-compatibility alias for USBContext.
